      subroutine grad24_p(dpx,dpy,j)
c
c=======================================================================
c                                                                    ===
c  This routine computes the horizontal pressure gradients for the   ===
c  current slab by first computing the density for the needed slab.  ===
c  The density values are then interpolated to  flat Z-levels  and   ===
c  the horizontal gradient of the density is formed.  Finally, the   ===
c  horizontal gradient of the density  is integrated vertically to   ===
c  get the horizontal pressure gradient.                             ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c      J         current slab, [2,jmt-2] (integer)                   ===
c                                                                    ===
c   Common Blocks:                                                   ===
c                                                                    ===
c     /ONEDIM/                                                       ===
c                                                                    ===
c     CSR     reciprocal cosine at velocity points.   (real vector)  ===
c     DXU2R   reciprocal of twice DXU.                (real vector)  ===
c     DYU2R   reciprocal of twice DYU.                (real vector)  ===
c     DXUR    reciprocal of DXU.                      (real vector)  ===
c     DYUR    reciprocal of DYU.                      (real vector)  ===
c                                                                    ===
#if defined surfpress & defined freesurf
c     /FIELDS/                                                       ===
c                                                                    ===
c     P       Surface pressure.          (real array; cm^2/s^2)      ===
c                                                                    ===
#endif
#ifdef rmdenbar
c     /RHOMEAN/                                                      ===
c                                                                    ===
c     RHOBAR     mean density, to be removed.   (real array)         ===
c                                                                    ===
#endif
c     /SCALAR/                                                       ===
c                                                                    ===
c     GRAV      gravitational acceleration.   (real; cm/(s^2))       ===
c                                                                    ===
#if defined surfpress & defined freesurf
c     /VERTICAL/                                                     ===
c                                                                    ===
c     ETAT    Surface elevation at t-points.   (real array; cm)      ===
c                                                                    ===
#endif
c     /VERTSLAB/                                                     ===
c                                                                    ===
c     DZQZ      thickness of the tracer box.               ===
c     DZV2RQZ   reciprocal of twice thickness of velocity box.
c     TDEPTH    depths at tracer points.  (real array; cm)      ===
c                                                                    ===
c     /WORKSP/                                                       ===
c                                                                    ===
c     T          temperature and salinity in slab J.   (real array)  ===
c     TP         T-S data in slab J+1.                 (real array)  ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c      DPX       x-component of the pressure gradient.   (real array)===
c      DPY       y-component of the pressure gradient.   (real array)===
c                                                                    ===
c   Common Blocks:                                                   ===
c                                                                    ===
c     /WORKSP/                                                       ===
c                                                                    ===
c     RHON       density in north slab.   (real array)               ===
c     RHOS       density in south slab.   (real array)               ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c    STATE                                                           ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <onedim.h>
#include <scalar.h>
#include <workspa.h>
#include <vertslabs.h>
#ifdef rmdenbar
# include <rhomean.h>
#endif
#if defined surfpress & defined freesurf
# include <fields.h>
# include <vertical.h>
#endif
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,j,k,ljrn,ljrs
      FLOAT
     &      dzp,dzp2,facx,facy,frcb,nrmx,nrmy,rgrdmg
      FLOAT
     &      dpx(imt,km),dpy(imt,km),etabot(imt,0:km,0:1),
     &      frcbx(imt,0:km),frcby(imt,0:km),frcsn(imt,km,0:1),
     &      frcw(imt,km),pbot(imt,0:km,0:1),rho(imt,km,0:1)
c
      save ljrn,ljrs,etabot,frcsn,pbot,rho
c
      data ljrs,ljrn /0,1/
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Update pointers.
c-----------------------------------------------------------------------
c
      ljrs = 1 - ljrs
      ljrn = 1 - ljrn
c
c-----------------------------------------------------------------------
c  Get new northernmost density slab.
c-----------------------------------------------------------------------
c
      if (j.eq.2) then
         call state(t(1,1,1),t(1,1,2),tdepth(1,1,jrs),rho(1,1,ljrs))
#ifdef rmdenbar
         do 10 k=1,km
         do 10 i=1,imt
            rho(i,k,ljrs)=rho(i,k,ljrs)-rhobar(i,j,k)
  10     continue
#endif
      endif
c
      call state(tp(1,1,1),tp(1,1,2),tdepth(1,1,jrn),rho(1,1,ljrn))
#ifdef rmdenbar
      do 20 k=1,km
      do 20 i=1,imt
         rho(i,k,ljrn)=rho(i,k,ljrn)-rhobar(i,j+1,k)
  20  continue
#endif
c
c-----------------------------------------------------------------------
c  Construct depths & pressures at bottom of boxes.
#if defined surfpress & defined freesurf
c  Remove any implicit surface pressure effects.  Surface pressure is
c  handled explicitly elsewhere in the code.
#endif
c-----------------------------------------------------------------------
c
      if (j.eq.2) then
         do 30 i = 1, imt
#if !defined surfpress | !defined freesurf
            etabot(i,0,ljrs) = c0
            pbot(i,0,ljrs)   = c0
#else
            etabot(i,0,ljrs) = etat(i,j)
# ifndef trytop
            pbot(i,0,ljrs)   = -grav*rho(i,1,ljrs)*etat(i,j)
# else
            pbot(i,0,ljrs)   = c0
# endif
c            pbot(i,0,ljrs)   = -p(i,j)
cc            pbot(i,0,ljrs)   = p(i,j)
#endif
c            pbot(i,0,ljrs)   = c0
  30     continue
         do 40 k=1,km
         do 40 i=1,imt
            etabot(i,k,ljrs)=etabot(i,k-1,ljrs)-dzqz(i,k,0)
            pbot(i,k,ljrs)=pbot(i,k-1,ljrs)
     &                     +grav*rho(i,k,ljrs)*dzqz(i,k,0)
  40     continue
      endif
c
      do 50 i = 1, imt
#if !defined surfpress | !defined freesurf
         etabot(i,0,ljrn) = c0
         pbot(i,0,ljrn)   = c0
#else
         etabot(i,0,ljrn) = etat(i,j+1)
# ifndef trytop
         pbot(i,0,ljrn)   = -grav*rho(i,1,ljrn)*etat(i,j+1)
# else
         pbot(i,0,ljrn)   = c0
# endif
c         pbot(i,0,ljrn)   = -p(i,j+1)
cc         pbot(i,0,ljrn)   = p(i,j+1)
#endif
c         pbot(i,0,ljrn)   = c0
  50  continue
      do 60 k=1,km
      do 60 i=1,imt
         etabot(i,k,ljrn)=etabot(i,k-1,ljrn)-dzqz(i,k,1)
         pbot(i,k,ljrn)=pbot(i,k-1,ljrn)+grav*rho(i,k,ljrn)*dzqz(i,k,1)
  60  continue
c
c-----------------------------------------------------------------------
c  Construct forces on north & south walls.  Note: a factor of 1/2 has
c  been "reserved" for the final assembly.
c-----------------------------------------------------------------------
c
      facx = grav*p25
c
      if (j.eq.2) then
         facy = cst(j)*r3
         do 70 k = 1, km
         do 70 i = 1, imtm1
            dzp = dzqz(i,k,0) + dzqz(i+1,k,0)
            dzp2 = dzp*dzp
            frcsn(i,k,ljrs) = facy*(pbot(i,k-1,ljrs)*(dzqz(i,k,0)+dzp)
     &                           +pbot(i+1,k-1,ljrs)*(dzqz(i+1,k,0)+dzp)
     &            +facx*(rho(i,k,ljrs)*(c2*dzqz(i,k,0)*dzqz(i,k,0)+dzp2)
     &         +rho(i+1,k,ljrs)*(dzp2+c2*dzqz(i+1,k,0)*dzqz(i+1,k,0)) ))
  70     continue
      endif
c
      facy = cst(j+1)*r3
      do 80 k = 1, km
      do 80 i = 1, imtm1
         dzp = dzqz(i,k,1) + dzqz(i+1,k,1)
         dzp2 = dzp*dzp
         frcsn(i,k,ljrn) = facy*(pbot(i,k-1,ljrn)*(dzqz(i,k,1)+dzp)
     &                          +pbot(i+1,k-1,ljrn)*(dzqz(i+1,k,1)+dzp)
     &         +facx*(rho(i,k,ljrn)*(c2*dzqz(i,k,1)*dzqz(i,k,1)+dzp2)
     &         +rho(i+1,k,ljrn)*(dzp2+c2*dzqz(i+1,k,1)*dzqz(i+1,k,1)) ))
  80  continue
c
c-----------------------------------------------------------------------
c  Construct forces on west walls.  Note: a factor of 1/2 has been
c  "reserved" for the final assembly.
c-----------------------------------------------------------------------
c
      facy = csr(j)*r3
      do 90 k = 1, km
      do 90 i = 1, imt
         dzp = dzqz(i,k,0) + dzqz(i,k,1)
         dzp2 = dzp*dzp
         frcw(i,k) = facy*(pbot(i,k-1,ljrs)*(dzqz(i,k,0)+dzp)
     &                    +pbot(i,k-1,ljrn)*(dzp+dzqz(i,k,1))
     &         +facx*(rho(i,k,ljrs)*(c2*dzqz(i,k,0)*dzqz(i,k,0)+dzp2)
     &               +rho(i,k,ljrn)*(dzp2+c2*dzqz(i,k,1)*dzqz(i,k,1)) ))
  90  continue
c
c-----------------------------------------------------------------------
c  Construct forces on box bottoms.  Note: a factor of 1/2 has been
c  "reserved" for the final assembly.
c-----------------------------------------------------------------------
c
      facx = csr(j)
      facy = dyu2r(j)
      do 100 k = 0, km
      do 100 i = 1, imtm1
         frcb = p5*(pbot(i,k,ljrs)+pbot(i,k,ljrn)
     &               +pbot(i+1,k,ljrs)+pbot(i+1,k,ljrn))
c
         nrmx = facx*((etabot(i,k,ljrs)-etabot(i+1,k,ljrs)) +
     &                (etabot(i,k,ljrn)-etabot(i+1,k,ljrn)) )*dxu2r(i)
         nrmy = facy*((etabot(i,k,ljrs)-etabot(i,k,ljrn)) +
     &                (etabot(i+1,k,ljrs)-etabot(i+1,k,ljrn)) )
         rgrdmg = c1/sqrt(c1+nrmx*nrmx+nrmy*nrmy)
         nrmx   = nrmx*rgrdmg
         nrmy   = nrmy*rgrdmg
c
         frcbx(i,k) = frcb*nrmx
         frcby(i,k) = frcb*nrmy
 100  continue
c
c-----------------------------------------------------------------------
c  Assemble net horizontal component of the pressure force.  Actually,
c  the negative of the net force, to agree with the sign in clinic.F
c-----------------------------------------------------------------------
c
      facy = csr(j)*dyur(j)
      do 110 k = 1, km
      do 110 i = 1, imtm1
#ifndef nullgradpin
        dpx(i,k) = ( (frcw(i+1,k)-frcw(i,k))*dxur(i) +
     &               (frcbx(i,k-1)-frcbx(i,k)) ) * dzv2rqz(i,k)
        dpy(i,k) = ( (frcsn(i,k,ljrn)-frcsn(i,k,ljrs))*facy +
     &               (frcby(i,k-1)-frcby(i,k)) ) * dzv2rqz(i,k)
#else
        dpx(i,k) = c0
        dpy(i,k) = c0
#endif
 110  continue
c
#ifdef cyclic
c-----------------------------------------------------------------------
c  Set Cyclic boundary conditions.
c-----------------------------------------------------------------------
c
      do 120 k=1,km
        dpx(1  ,k)=dpx(imum1,k)
        dpy(1  ,k)=dpy(imum1,k)
        dpx(imu,k)=dpx(2    ,k)
        dpy(imu,k)=dpy(2    ,k)
 120  continue
c
#endif
c-----------------------------------------------------------------------
c  Maintain density arrays RHON and RHOS.
c-----------------------------------------------------------------------
c
      do 130 k=1,km
      do 130 i=1,imt
        rhon(i,k)=rho(i,k,ljrn)
        rhos(i,k)=rho(i,k,ljrs)
 130  continue
c
      return
      end
