      subroutine gradconv (ncmpct,pcmpct,atol,rtol,isconv)
c
c=======================================================================
c                                                                    ===
c  This routine examines the current pressure tendency to see if its ===
c  gradients have converged to within the requested tolerences.      ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     NCMPCT   Number of active pressure points.          (integer)  ===
c     PCMPCT   Surface pressure change, compact storage.  (dble prec)===
c     ATOL     Convergence absolute tolerence.            (dble prec)===
c     RTOL     Convergence relative tolerence.            (dble prec)===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
#ifdef coast
c  /FULLWD/                                                          ===
c                                                                    ===
c     IEP   Ending indices for surface pressure.     (integer array) ===
c     IEQ   Ending indices for velocity points.      (integer array) ===
c     ISP   Starting indices for surface pressure.   (integer array) ===
c     ISQ   Starting indices for velocity points.    (integer array) ===
c                                                                    ===
#endif
c  /ONEDIM/                                                          ===
c                                                                    ===
c     CSR     reciprocal of cosine, velocity points.   (real vector) ===
c     DXU2R   reciprocal of twice velocity x-spacing.  (real vector) ===
c     DYU2R   reciprocal of twice velocity y-spacing.  (real vector) ===
c                                                                    ===
c  /WORKSPB/                                                         ===
c                                                                    ===
c     PTD     time rate of change of pressure (BCs).   (real array)  ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     ISCONV   Flag to indicate convergence.               (logical) ===
c              [true]   gradients have converged.                    ===
c              [false]  gradients have not yet converged.            ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#ifdef coast
# include <fullwd.h>
#endif
#include <onedim.h>
#include <workspb.h>
#if defined sprsnrm01 & defined sprsnrm02 & defined sprsnrm05
# include <vertslabs.h>
#endif
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iend,istr,j,knew,kold,n,ncmpct
#ifdef coast
      integer jptr,l
#endif
      logical first,isconv
      double precision 
     &      atol,rtol,testval
      double precision 
     &      dcsr(jmt),ddxu2r(imt),ddyu2r(jmt),dpsdt(imt,jmt),
     &      d2psdxdt(imt,jmt,0:1),d2psdydt(imt,jmt,0:1),pcmpct(ncmpct)
c
      save first,knew,kold,dcsr,ddxu2r,ddyu2r,d2psdxdt,d2psdydt
c
      data knew,kold /0,1/
      data first /.true./
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Transfer change in surface pressure from compact storage to 2D.
c-----------------------------------------------------------------------
c
      n    = 0
      istr = 2
      iend = imtm1
c
      do 30 j = 2, jmtm1
# ifdef coast
         do 20 l = 1, lseg
            istr = isp(j,l)
            if(istr.eq.0) go to 30
            iend = iep(j,l)
# endif
            do 10 i = istr, iend
               n = n+1
               dpsdt(i,j)=pcmpct(n)
  10        continue
# ifdef coast
  20     continue
# endif
  30  continue
c
      do 40, i = 2, imtm1
         dpsdt(i,1)   = ptd(i,1)
         dpsdt(i,jmt) = ptd(i,jmt)
  40  continue
      do 50, j = 1, jmt
         dpsdt(1,j)   = ptd(1,j)
         dpsdt(imt,j) = ptd(imt,j)
  50  continue
c
c-----------------------------------------------------------------------
c  Examine change in gradients of surface pressure tendency.
c-----------------------------------------------------------------------
c
c  Rotate storage pointers.
c
      knew = 1 - knew
      kold = 1 - knew
      testval = c0
      istr = 1
      iend = imtm1
c
c  Loop over all active velocity points, examining gradient.
c
      do 80 j = 1, jmtm1
#if defined coast & !(defined sprsnrm01 & defined sprsnrm02 & defined sprsnrm05)
         jptr = min(max(j,2),jmtm2)
         do 70 l=1,lseg
            istr=isq(jptr,l)
            if(istr.eq.0) go to 80
            iend=ieq(jptr,l)
            if(istr.eq.2)     istr=1
            if(iend.eq.imtm2) iend=imtm1
#endif
            do 60 i = istr, iend
c
#if !(defined sprsnrm01 & defined sprsnrm02 & defined sprsnrm05)
               d2psdxdt(i,j,knew) = ((dpsdt(i+1,j+1)-dpsdt(i,j+1)) +
     &                               (dpsdt(i+1,j)-dpsdt(i,j)))
     &                              *dcsr(j)*ddxu2r(i)
c
               d2psdydt(i,j,knew) = ((dpsdt(i+1,j+1)-dpsdt(i+1,j)) +
     &                               (dpsdt(i,j+1)-dpsdt(i,j)))
     &                              *ddyu2r(j)
#else
               if (uland(i,j).gt.0) then
                  d2psdxdt(i,j,knew) = ((dpsdt(i+1,j+1)-dpsdt(i,j+1))
#  if defined coast
     &                                 *min(landt(i+1,j+1),landt(i,j+1))
#  endif
     &                                  + (dpsdt(i+1,j)-dpsdt(i,j))
#  if defined coast
     &                                 *min(landt(i+1,j),landt(i,j))
#  endif
     &                                 )*dcsr(j)*ddxu2r(i)
# if defined sprsnrm04
     &                                 *ugdwt(i,j)
# endif
               else
                  d2psdxdt(i,j,knew) = c0
               endif
c
               if (vland(i,j).gt.0) then
                  d2psdydt(i,j,knew) = ((dpsdt(i+1,j+1)-dpsdt(i+1,j))
#  if defined coast
     &                                 *min(landt(i+1,j+1),landt(i+1,j))
#  endif
     &                                  + (dpsdt(i,j+1)-dpsdt(i,j))
#  if defined coast
     &                                 *min(landt(i,j+1),landt(i,j))
#  endif
     &                                 )*ddyu2r(j)
# if defined sprsnrm04
     &                                 *vgdwt(i,j)
# endif
               else
                  d2psdydt(i,j,knew) = c0
               endif
#endif
c
               testval = max(testval,
     &                       (abs(d2psdxdt(i,j,knew)-d2psdxdt(i,j,kold))
     &                        -(rtol*abs(d2psdxdt(i,j,knew))+atol)),
     &                       (abs(d2psdydt(i,j,knew)-d2psdydt(i,j,kold))
     &                        -(rtol*abs(d2psdydt(i,j,knew))+atol)) )
c
  60        continue
#ifdef coast
  70     continue
# endif
  80  continue
c
      isconv = testval .le. c0
c
      return
c
c-----------------------------------------------------------------------
c  Entry to initialize rolling storage.
c-----------------------------------------------------------------------
c
      entry gcinit
c
      do 110 n = 0, 1
         do 100 j = 1, jmt
            do 90 i = 1, imt
               d2psdxdt(i,j,n) = c0
               d2psdydt(i,j,n) = c0
  90        continue
 100     continue
 110  continue
c
      if (first) then
         do 120 i = 1, imt
            ddxu2r(i) = dble(dxu2r(i))
 120     continue
         do 130 j = 1, jmt
            dcsr(j)   = dble(csr(j))
            ddyu2r(j) = dble(dyu2r(j))
 130     continue
         first = .false.
      end if
c
      return
      end
