      subroutine nest_flags
c
c=======================================================================
c                                                                    ===
c This routine exchanges the domain data between nested runs.        ===
c                                                                    ===
c ------                                                             ===
c Input:                                                             ===
c ------                                                             ===
c                                                                    ===
c Common Blocks                                                      ===
c                                                                    ===
c /NEST/                                                             ===
c                                                                    ===
c    CURRTID   PVM task identifier for this run.       (integer)     ===
c    LRGTID    PVM task identifier for larger grid.    (integer)     ===
c    NOSUBD    number of nested sub-domains.           (integer)     ===
c    SMLTID    PVM task identifier for smaller grid.   (integer)     ===
c                                                                    ===
c /OPTIONS/                                                          ===
c                                                                    ===
c  IOPT    various switches from standard input:     (integer vector)===
c          IOPT(8)  Number of tracers exchanged with larger domain.  ===
c          IOPT(9)  Number of tracers exchanged with smaller domains.===
c                                                                    ===
c -------                                                            ===
c Output:                                                            ===
c -------                                                            ===
c                                                                    ===
c Common Blocks                                                      ===
c                                                                    ===
c /NEST/                                                             ===
c                                                                    ===
c    SPVL      Flag value from larger grid.               (real)     ===
c    SPVS      Flag value from smaller grid.              (real)     ===
#ifdef coast
c    T_ACT     Active value of land mask over tracer                 ===
c              points.                                    (integer)  ===
c    V_ACT     Active value of land mask over velocity               ===
c              points.                                    (integer)  ===
#endif
c                                                                    ===
c  Calls:      EXITUS,        HOPSRECV,  NEST_ERRCHK                 ===
c  PVM Calls:  PVMFINITSEND,  PVMFPACK,  PVMFSEND,    PVMFUNPACK,    ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#ifdef use_mpi
# include <mpif.h>
#else
# include <fpvm3.h>
#endif
#include <iounits.h>
#include <nest.h>
#include <options.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer bufid,lgind,n,ntpass,status
      integer testrec(mdomp1),xrec(mdomp1)
# ifdef use_mpi
      integer recvstat(MPI_STATUS_SIZE)
# endif
      logical nok,ntbok,nvbok,ok
c
#if defined nest2larger & defined nest2smaller
      parameter (lgind=mdomp1)
#else
      parameter (lgind=1)
#endif
c
      data nok,ntbok,nvbok /.true.,.true.,.true./
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
#ifdef coast
c-----------------------------------------------------------------------
c  Set up active values of land masks.
c-----------------------------------------------------------------------
c
      t_act = 1
      v_act = km
c
#endif
#ifdef nest2larger
c-----------------------------------------------------------------------
c  Exchange flag values with larger grid.
c-----------------------------------------------------------------------
c
# ifdef use_mpi
      call MPI_SEND (spv,1,nstflt,lrgtid,spvl2l,comm_nest,status)
# else
      call pvmfinitsend (PVM_DATAMETHOD,bufid)
      call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
      call pvmfpack (nstflt,spv,1,1,status)
      call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
      call pvmfsend (lrgtid,spvl2l,status)
      call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
# ifdef use_mpi
      call MPI_RECV (spvl,1,nstflt,lrgtid,
     &               spvl2s,comm_nest,recvstat,status)
# else
      call hopsrecv ('NEST_FLAGS',lrgtid,spvl2s,bufid)
      call pvmfunpack (nstflt,spvl,1,1,status)
      call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
c-----------------------------------------------------------------------
c  Determine the number of tracers to pass to larger grid.
c-----------------------------------------------------------------------
c
# ifdef use_mpi
      call MPI_SEND (iopt(8),1,MPI_INTEGER4,lrgtid,ntrc2l,comm_nest,
     &               status)
# else
      call pvmfinitsend (PVM_DATAMETHOD,bufid)
      call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
      call pvmfpack (INTEGER4,iopt(8),1,1,status)
      call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
      call pvmfsend (lrgtid,ntrc2l,status)
      call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
# ifdef use_mpi
      call MPI_RECV (ntpass,1,MPI_INTEGER4,lrgtid,
     &               ntrc2s,comm_nest,recvstat,status)
# else
      call hopsrecv ('NEST_FLAGS',lrgtid,ntrc2s,bufid)
      call pvmfunpack (INTEGER4,ntpass,1,1,status)
      call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
      nok = ntpass .eq. iopt(8)
c
      if (.not.nok)
     &   write (stdout,900) 'tracers','tracers',iopt(8),'tracers',
     &                      'larger',ntpass
c
c-----------------------------------------------------------------------
c  Request number of rows normal to boundary to receive from larger grid.
c-----------------------------------------------------------------------
c
c  Tracer normal rows.
c
# ifdef use_mpi
      call MPI_SEND (nnmtb,1,MPI_INTEGER4,lrgtid,nnmtb2l,comm_nest,
     &               status)
# else
      call pvmfinitsend (PVM_DATAMETHOD,bufid)
      call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
      call pvmfpack (INTEGER4,nnmtb,1,1,status)
      call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
      call pvmfsend (lrgtid,nnmtb2l,status)
      call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
# ifdef use_mpi
      call MPI_RECV (ntpass,1,MPI_INTEGER4,lrgtid,
     &               nnmtb2s,comm_nest,recvstat,status)
# else
      call hopsrecv ('NEST_FLAGS',lrgtid,nnmtb2s,bufid)
      call pvmfunpack (INTEGER4,ntpass,1,1,status)
      call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
      ntbok = ntpass .eq. nnmtb
c
      if (.not.ntbok)
     &   write (stdout,900) 'tracer boundary rows','rows',nnmtb,'rows',
     &                      'larger',ntpass
c
c  Velocity normal rows.
c
# ifdef use_mpi
      call MPI_SEND (nnmvb,1,MPI_INTEGER4,lrgtid,nnmvb2l,comm_nest,
     &               status)
# else
      call pvmfinitsend (PVM_DATAMETHOD,bufid)
      call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
      call pvmfpack (INTEGER4,nnmvb,1,1,status)
      call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
      call pvmfsend (lrgtid,nnmvb2l,status)
      call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
# ifdef use_mpi
      call MPI_RECV (ntpass,1,MPI_INTEGER4,lrgtid,
     &               nnmvb2s,comm_nest,recvstat,status)
# else
      call hopsrecv ('NEST_FLAGS',lrgtid,nnmvb2s,bufid)
      call pvmfunpack (INTEGER4,ntpass,1,1,status)
      call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
      nvbok = ntpass .eq. nnmvb
c
      if (.not.nvbok)
     &   write (stdout,900) 'velocity boundary rows','rows',nnmtb,'rows'
     &                      ,'larger',ntpass
c
#endif
#ifdef nest2smaller
c-----------------------------------------------------------------------
c  Exchange flag values with smaller grid.
c-----------------------------------------------------------------------
c
      do 10, n = 1, nosubd
# ifdef use_mpi
         call MPI_RECV (spvs(n),1,nstflt,smltid(n),
     &                  spvl2l,comm_nest,recvstat,status)
# else
         call hopsrecv ('NEST_FLAGS',smltid(n),spvl2l,bufid)
         call pvmfunpack (nstflt,spvs(n),1,1,status)
         call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
# ifdef use_mpi
         call MPI_SEND (spv,1,nstflt,smltid(n),spvl2s,comm_nest,
     &                  status)
# else
         call pvmfinitsend (PVM_DATAMETHOD,bufid)
         call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
         call pvmfpack (nstflt,spv,1,1,status)
         call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
         call pvmfsend (smltid(n),spvl2s,status)
         call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
c-----------------------------------------------------------------------
c  Determine the number of tracers to pass to smaller grid.
c-----------------------------------------------------------------------
c
# ifdef use_mpi
         call MPI_RECV (ntpass,1,MPI_INTEGER4,smltid(n),
     &                  ntrc2l,comm_nest,recvstat,status)
# else
         call hopsrecv ('NEST_FLAGS',smltid(n),ntrc2l,bufid)
         call pvmfunpack (INTEGER4,ntpass,1,1,status)
         call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
# ifdef use_mpi
         call MPI_SEND (iopt(9),1,MPI_INTEGER4,smltid(n),ntrc2s,
     &                  comm_nest,status)
# else
         call pvmfinitsend (PVM_DATAMETHOD,bufid)
         call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
         call pvmfpack (INTEGER4,iopt(9),1,1,status)
         call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
         call pvmfsend (smltid(n),ntrc2s,status)
         call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
         nok = nok .and. (ntpass .eq. iopt(9))
c
c-----------------------------------------------------------------------
c  Determine number of rows normal to boundary to pass to smaller grid.
c-----------------------------------------------------------------------
c
c  Tracer normal rows.
c
# ifdef use_mpi
         call MPI_RECV (nnmtbs(n),1,MPI_INTEGER4,smltid(n),
     &                  nnmtb2l,comm_nest,recvstat,status)
# else
         call hopsrecv ('NEST_FLAGS',smltid(n),nnmtb2l,bufid)
         call pvmfunpack (INTEGER4,nnmtbs(n),1,1,status)
         call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
# ifdef use_mpi
         call MPI_SEND (nnmtbs(n),1,MPI_INTEGER4,smltid(n),nnmtb2s,
     &                  comm_nest,status)
# else
         call pvmfinitsend (PVM_DATAMETHOD,bufid)
         call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
         call pvmfpack (INTEGER4,nnmtbs(n),1,1,status)
         call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
         call pvmfsend (smltid(n),nnmtb2s,status)
         call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
c
c  Velocity normal rows.
c
# ifdef use_mpi
         call MPI_RECV (nnmvbs(n),1,MPI_INTEGER4,smltid(n),
     &                  nnmvb2l,comm_nest,recvstat,status)
# else
         call hopsrecv ('NEST_FLAGS',smltid(n),nnmvb2l,bufid)
         call pvmfunpack (INTEGER4,nnmvbs(n),1,1,status)
         call nest_errchk ('NEST_FLAGS','UnPack',status,1,1,1)
# endif
c
# ifdef use_mpi
         call MPI_SEND (nnmvbs(n),1,MPI_INTEGER4,smltid(n),nnmvb2s,
     &                  comm_nest,status)
# else
         call pvmfinitsend (PVM_DATAMETHOD,bufid)
         call nest_errchk ('NEST_FLAGS','InitSend',bufid,1,1,1)
         call pvmfpack (INTEGER4,nnmvbs(n),1,1,status)
         call nest_errchk ('NEST_FLAGS','Pack',status,1,1,1)
         call pvmfsend (smltid(n),nnmvb2s,status)
         call nest_errchk ('NEST_FLAGS','Send',status,1,1,1)
# endif
  10  continue
c
      if (.not.nok)
     &   write (stdout,900) 'tracers','tracers',iopt(9),'tracers',
     &                      'smaller',ntpass
c
#endif
c-----------------------------------------------------------------------
c  Make sure there is agreement on the passing of data.
c-----------------------------------------------------------------------
c
#ifdef nest2smaller
      do 20, n = 1, nosubd
         xrec(n)     = smltid(n)
  20  continue
#endif
#ifdef nest2larger
      xrec(lgind) = lrgtid
#endif
c
c Number of tracers.
c
      if (nok) then
         call nest_test (currtid,xrec,ok,testrec)
         if (.not. ok) write (stdout,910) 'tracer'
        else
         call nest_test (currtid-1,xrec,ok,testrec)
      end if
c
c Number of tracer normal rows.
c
      if (ntbok) then
         call nest_test (currtid,xrec,ok,testrec)
         if (.not. ok) write (stdout,910) 'tracer normal rows'
        else
         call nest_test (currtid-1,xrec,ok,testrec)
      end if
c
c Number of velocity normal rows.
c
      if (nvbok) then
         call nest_test (currtid,xrec,ok,testrec)
         if (.not. ok) write (stdout,910) 'velocity normal rows'
        else
         call nest_test (currtid-1,xrec,ok,testrec)
      end if
c
c Concatenated error checks.
c
      if (.not. (nok.and.ntbok.and.nvbok.and.ok) )
     &   call exitus ('NEST_FLAGS')
c
      return
c
 900  format (/'***Error:  NEST_FLAGS - inconsistent number of passed ',
     &        a/11x,'Number ',a,' (current domain): ',i10
     &        /11x,'Number ',a,' (',a,' domain): ',i10)
 910  format (/'***Error:  NEST_FLAGS - disagreement on ',a,' passing',
     &        ' in another node in the nesting chain.')
c
      end
