      subroutine restrtps (p,pb)
c
c=======================================================================
c                                                                    ===
c  This routine attempts to ensure that after assimilation, the rate ===
c  of change of surface pressure (and hence initial rate of          ===
c  deformation of free surface) is consistent with the divergence    ===
c  used in the model.                                                ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     P    Surface pressure at time 0.   (real array)                ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /FIELDSBAR/                                                       ===
c                                                                    ===
#if defined transbc
c     FKMQ     number of vertical levels of ocean at U,V points.     ===
#endif
c     UBARO   initial zonal external mode velocity.      (real array)===
c     VBARO   initial meridional external mode velocity. (real array)===
c                                                                    ===
c  /ONEDIM/                                                          ===
c                                                                    ===
c     CS      cosine metric factors at velocity points.      (real)  ===
c     CST     cosine metric factors at tracer points.        (real)  ===
c     CSTR    reciprocal of CST (CSTR=1/CST).                (real)  ===
c     DXT4R   reciprocal of four times DXT (DXT4R=0.25/DXT). (real)  ===
c     DXU     zonal grid spacing across velocity boxes.      (real)  ===
c     DYT     meridional grid spacing across tracer boxes.   (real)  ===
c     DYTR    reciprocal of DYT (DYTR=1/DYTR).               (real)  ===
c     DYU     meridional grid spacing across velocity boxes. (real)  ===
c                                                                    ===
c  /SCALAR/                                                          ===
c                                                                    ===
c     DTSF   timestep on surface pressure.   (real; s)
c                                                                    ===
# if defined transbc
c  /VERTSLABS/                                                       ===
c                                                                    ===
c     HDV     Topography at V-points.                    (real; cm)  ===
c                                                                    ===
# endif
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     PB   Surface pressure at time -dt.   (real array)              ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /WORKSPB/                                                         ===
c                                                                    ===
c     PTD    time rate of change of surface pressure.   (real array) ===
c                                                                    ===
#if defined transbc | defined oias
c  /VERTICAL/                                                        ===
c                                                                    ===
c     ETAT    Surface elevation at t-points, time -dt.   (real; cm)  ===
c     ETAV    Surface elevation at v-points, time -dt.   (real; cm)  ===
c                                                                    ===
# if defined transbc
c  /VERTSLABS/                                                       ===
c                                                                    ===
c     HDVCF   Water column depth at V-points, time -dt.  (real; cm)  ===
c                                                                    ===
# endif
#endif
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     SETVERT                                                        ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <fieldsbar.h>
#include <onedim.h>
#include <scalar.h>
#include <vertslabs.h>
#include <workspa.h>
#include <workspb.h>
#ifdef ext_tide
# include <fullwd.h>
#endif
#if defined transbc | defined oias
# include <vertical.h>
#endif
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iu,jj,jjm1,k
      FLOAT
     &      fxa,fxb,gdt
      FLOAT
     &      p(imt,jmt),pb(imt,jmt)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c   Initialize pressure tendency by divergence of barotropic velocity.
c-----------------------------------------------------------------------
c
c  Establish over dimensioned arrays for vectorization.
c
      do 10 k=1,km
      do 10 i=1,imt
        dxt4rq(i,k)=dxt4r(i)
        dxuq  (i,k)=dxu  (i)
  10  continue
c
c  Compute barotropic component of fluxes through southernmost wall.
c
      call setvert(1)
      fxb  = p5*cstr(2)*dytr(2)*cs(1)
      do 20 k=1,km
      do 20 i=2,imt
        iu = min(i,imu)
        fvst(i,k)=((vbaro(iu,1)+vbarob(iu,1))*dxuq(i,k)*dzvqzb(iu,k,0)
     &            +(vbaro(i-1,1)+vbarob(i-1,1))*dxuq(i-1,k)*
     &            dzvqzb(i-1,k,0)) * fxb*dxt4rq(i,k)
  20  continue
c
c  Save scaling between surface vertical flux and initial pressure tendency.
c
      gdt = c2dtsf*grav
c
c  Loop over active rows, computing divergence.  Include row JMT-1 so
c  DETAV is well defined through row JMT-2
c
      do 90 jj = 2, jmtm1
c
c  Compute barotropic component of fluxes through walls.
c
         call setvert(jj)
         fxa  = p5*cstr(jj)*dytr(jj)
         fxb  = fxa*cs(jj)
         jjm1 = max((jj-1),1)
c
         do 30 k=1,km
         do 30 i=2,imt
           iu = min(i,imu)
           fuw(i,k)=((ubaro(i-1,jj)+ubarob(i-1,jj))*dyu(jj)
     &               *dzvqzb(i-1,k,0)+
     &               (ubaro(i-1,jjm1)+ubarob(i-1,jjm1))*dyu(jjm1)
     &               *dzvqzb(i-1,k,1))*fxa
           fvn(i,k)=((vbaro(iu,jj)+vbarob(iu,jj))*dxuq(i  ,k)
     &               *dzvqzb(iu ,k,0)+
     &               (vbaro(i-1,jj)+vbarob(i-1,jj))*dxuq(i-1,k)
     &               *dzvqzb(i-1,k,0)) * fxb*dxt4rq(i,k)
  30     continue
c
c  Set W at the bottom.  Kinematic boundary condition
c
         do 40 i=2,imtm1
           w(i,km+1)=c0
  40     continue
c
c  Compute change of W between levels.
c
         do 50 k=1,km
         do 50 i=2,imtm1
           w(i,k)=c2*((fuw(i+1,k)-fuw (i,k))*dxt4rq(i,k)
     &                  + (fvn(i  ,k)-fvst(i,k)) )
  50     continue
c
c  Integrate upwards from the bottom
c
         do 60 k=km,1,-1
         do 60 i=2,imtm1
           w(i,k)=-w(i,k)+w(i,k+1)
  60     continue
c
c  The surface value of W is rescaled to be the initial pressure tendency
c
         do 70 i=2,imtm1
           ptd(i,jj)=gdt*w(i,1)
  70     continue
c
c  Transfer quantities computed to the north of the present row
c  to be defined to the south in the computation of the next row
c
         fxa=cst(jj)*dyt(jj)*cstr(jj+1)*dytr(jj+1)
         do 80 k=1,km
         do 80 i=1,imt
           fvst(i,k)=fvn(i,k)*fxa
  80     continue
c
  90  continue
c
c-----------------------------------------------------------------------
c  Initialize pressure at time -dt by pressure tendency.
c-----------------------------------------------------------------------
c
      do 100 jj = 1, jmt
      do 100 i = 1, imt
         pb(i,jj) = p(i,jj) - ptd(i,jj)
 100  continue
c
      return
      end
