#if defined  surfpress & defined freesurf & (defined rset_dps | defined smallpsdt3 | defined specpsic)
      subroutine start_ps (p,pb,ztd)
#else
      subroutine start_ps (p,pb)
#endif
c
c=======================================================================
c                                                                    ===
c  This routine attempts to ensure that the initial rate of change   ===
c  of surface pressure (and hence initial rate of deformation of     ===
c  free surface) is consistent with the divergence used in the model.===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     P    Surface pressure at time 0.   (real array)                ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /FIELDSBAR/                                                       ===
c                                                                    ===
#if defined transbc
c     FKMQ     number of vertical levels of ocean at U,V points.     ===
#endif
c     UBARO   initial zonal external mode velocity.      (real array)===
c     VBARO   initial meridional external mode velocity. (real array)===
c                                                                    ===
c  /ONEDIM/                                                          ===
c                                                                    ===
c     CS      cosine metric factors at velocity points.      (real)  ===
c     CST     cosine metric factors at tracer points.        (real)  ===
c     CSTR    reciprocal of CST (CSTR=1/CST).                (real)  ===
c     DXT4R   reciprocal of four times DXT (DXT4R=0.25/DXT). (real)  ===
c     DXU     zonal grid spacing across velocity boxes.      (real)  ===
c     DYT     meridional grid spacing across tracer boxes.   (real)  ===
c     DYTR    reciprocal of DYT (DYTR=1/DYTR).               (real)  ===
c     DYU     meridional grid spacing across velocity boxes. (real)  ===
c                                                                    ===
c  /SCALAR/                                                          ===
c                                                                    ===
c     DTSF   timestep on surface pressure.   (real; s)
c                                                                    ===
# if defined transbc
c  /VERTSLABS/                                                       ===
c                                                                    ===
c     HDV     Topography at V-points.                    (real; cm)  ===
c                                                                    ===
# endif
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     PB   Surface pressure at time -dt.   (real array)              ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /WORKSPB/                                                         ===
c                                                                    ===
c     PTD    time rate of change of surface pressure.   (real array) ===
c                                                                    ===
#if defined transbc | defined oias
c  /VERTICAL/                                                        ===
c                                                                    ===
c     ETAT    Surface elevation at t-points, time -dt.   (real; cm)  ===
c     ETAV    Surface elevation at v-points, time -dt.   (real; cm)  ===
c                                                                    ===
# if defined transbc
c  /VERTSLABS/                                                       ===
c                                                                    ===
c     HDVCF   Water column depth at V-points, time -dt.  (real; cm)  ===
c                                                                    ===
# endif
#endif
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     SETVERT                                                        ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <fieldsbar.h>
#include <onedim.h>
#include <scalar.h>
#include <vertslabs.h>
#include <workspa.h>
#include <workspb.h>
#ifdef ext_tide
# include <fullwd.h>
# ifdef tidepsic
#  include <tidesp.h>
# endif
#endif
#if defined transbc | defined oias
# include <vertical.h>
#endif
#if surfpress & defined freesurf & (defined rset_dps | defined smallpsdt3 | defined specpsic)
# include <sinfo.h>
#endif
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iu,jj,jjm1,k
      FLOAT
     &      fxa,fxb,gdt
      FLOAT
     &      p(imt,jmt),pb(imt,jmt)
#if surfpress & defined freesurf & (defined rset_dps | defined smallpsdt3 | defined specpsic)
     &      ,uf(imt,jmt),vf(imt,jmt),ztd(imt,jmt)
#endif
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
#ifdef ext_tide
c-----------------------------------------------------------------------
c  Initialize tidal elevation data.
c-----------------------------------------------------------------------
c
      ttsec = -dtts
      call get_tide (1)
c
      ttsec = c0
      call get_tide (1)
# ifdef tide_pert
      call get_perttide
# endif
c
#endif
#if !defined nullps & !defined psnull
# if !defined tidepsic | !defined ext_tide
#  if surfpress & defined freesurf & (defined rset_dps | defined smallpsdt3 | defined specpsic)
c-----------------------------------------------------------------------
c  Initialize pressure tendency by divergence of barotropic velocity.
c-----------------------------------------------------------------------
c
c  Initialize "new" barotropic velocity to initial condition
c
      fxa=c1/grav
      do 2 jj=1,jmt
      do 2 i=1,imt
         etat(i,jj)=fxa*p(i,jj)
   2  continue
      do 6 jj=1,jmtm1
      do 4 i=1,imtm1
          etav(i,jj)=p25*
     &          (etat(i,jj)+etat(i+1,jj)+etat(i,jj+1)+etat(i+1,jj+1))
   4    continue
        etav(imt,jj)=etav(imtm1,jj)
   6  continue
      do 8 i=1,imt
        etav(i,jmt)=etav(i,jmtm1)
   8  continue
      do 10 jj=1,jmt
      do 10 i=1,imt
         ubaronew(i,jj) = ubaro(i,jj)
         vbaronew(i,jj) = vbaro(i,jj)
         hdvcf(i,jj)    = hdv(i,jj)+etav(i,jj)
         hdvcf(i,jj)    = hdvcf(i,jj)*min(fkmq(i,jj),c1)
  10  continue
c
      c2dtps=c2*dtsf
c
c  Initialize pressure tendency by divergence of barotropic velocity.
c
      call h_frc_rst (uf,vf,fxa)
#   if !defined sprsnrm01 | !defined sprsnrm02
      call div_fld (uf,vf,ztd,hdvcf,fxa)
#   else
      call div_fld (uf,vf,ztd,hdvcfx,hdvcfy,fxa)
#   endif
      call reset_ps (.true.)
c
#   ifdef dumppsrst
         open (413, file='facIC.dat', status='unknown')
         write (413,*) fxa
         close(413)
         open (413, file='pIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) p(i,j)
            enddo
         enddo
         close(413)
         open (413, file='ufIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) uf(i,j)
            enddo
         enddo
         close(413)
         open (413, file='vfIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) vf(i,j)
            enddo
         enddo
         close(413)
#   if !defined sprsnrm01 | !defined sprsnrm02
         open (413, file='hdvcfIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) hdvcf(i,j)
            enddo
         enddo
         close(413)
#   else
         open (413, file='hdvcfxIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) hdvcfx(i,j)
            enddo
         enddo
         close(413)
         open (413, file='hdvcfyIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) hdvcfy(i,j)
            enddo
         enddo
         close(413)
#   endif
         open (413, file='ubaronewIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) ubaronew(i,j)
            enddo
         enddo
         close(413)
         open (413, file='vbaronewIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) vbaronew(i,j)
            enddo
         enddo
         close(413)
         open (413, file='ubaroIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) ubaro(i,j)
            enddo
         enddo
         close(413)
         open (413, file='vbaroIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) vbaro(i,j)
            enddo
         enddo
         close(413)
         open (413, file='ubarobIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) ubarob(i,j)
            enddo
         enddo
         close(413)
         open (413, file='vbarobIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) vbarob(i,j)
            enddo
         enddo
         close(413)
         open (413, file='ztd2IC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) ztd(i,j)
            enddo
         enddo
         close(413)
         open (413, file='ptdIC.dat', status='unknown')
         do j = 1, jmt
            do i = 1, imt
               write (413,*) ptd(i,j)
            enddo
         enddo
         close(413)
c
#   endif
c  Extend pressure tendency to boundaries
c
      do 20 jj=2,jmtm1
#   ifndef cyclic
         ptd(1  ,jj)=ptd(2    ,jj)
         ptd(imt,jj)=ptd(imtm1,jj)
#   else
         ptd(1  ,jj)=ptd(imtm1,jj)
         ptd(imt,jj)=ptd(2    ,jj)
#   endif
  20  continue
c
      do 30 i=1,imt
         ptd(i,1  )=ptd(i,2)
         ptd(i,jmt)=ptd(i,jmtm1)
  30  continue
c
      do 60, iu = 1, 1
         fxa=c1/grav
         do 35 jj = 1, jmt
         do 35 i = 1, imt
            pb(i,jj) = p(i,jj) - p5*ptd(i,jj)
            etat(i,jj)=fxa*pb(i,jj)
  35     continue
         do 45 jj=1,jmtm1
         do 40 i=1,imtm1
             etav(i,jj)=p25*
     &             (etat(i,jj)+etat(i+1,jj)+etat(i,jj+1)+etat(i+1,jj+1))
  40       continue
           etav(imt,jj)=etav(imtm1,jj)
  45     continue
         do 50 i=1,imt
           etav(i,jmt)=etav(i,jmtm1)
  50     continue
         do 55 jj=1,jmt
         do 55 i=1,imt
            hdvcf(i,jj)    = hdv(i,jj)+etav(i,jj)
            hdvcf(i,jj)    = hdvcf(i,jj)*min(fkmq(i,jj),c1)
  55     continue
c
         call h_frc_rst (uf,vf,fxa)
#   if !defined sprsnrm01 | !defined sprsnrm02
         call div_fld (uf,vf,ztd,hdvcf,fxa)
#   else
         call div_fld (uf,vf,ztd,hdvcfx,hdvcfy,fxa)
#   endif
         call reset_ps (.true.)
  60  continue
#   if defined smallpsdt & defined smallpsdt4
      do 70 jj=1,jmtm1
         fx                  = p5*(c1+etav(i,jj)/hdv(i,jj))
         uwkadv(i,jj,iptadv) = fx*uf(i,jj)
         vwkadv(i,jj,iptadv) = fx*vf(i,jj)
         ptdwk(i,jj,iptadv) = p5*ptd(i,jj)
  70  continue
#   endif
c
#  else
c-----------------------------------------------------------------------
c  Initialize pressure tendency by divergence of barotropic velocity.
c-----------------------------------------------------------------------
c
c  Establish over dimensioned arrays for vectorization.
c
      do 10 k=1,km
      do 10 i=1,imt
        dxt4rq(i,k)=dxt4r(i)
        dxuq  (i,k)=dxu  (i)
  10  continue
c
c  Compute barotropic component of fluxes through southernmost wall.
c
      call setvert(1)
      fxb  = cstr(2)*dytr(2)*cs(1)
      do 20 k=1,km
      do 20 i=2,imt
        iu = min(i,imu)
        fvst(i,k)=(vbaro(iu,1)*dxuq(i  ,k)*dzvqz(iu ,k,0)+
     &             vbaro(i-1,1)*dxuq(i-1,k)*dzvqz(i-1,k,0))*fxb
     &            *dxt4rq(i,k)
  20  continue
c
c  Save scaling between surface vertical flux and initial pressure tendency.
c
      gdt = dtsf*grav
c
c  Loop over active rows, computing divergence.  Include row JMT-1 so
c  DETAV is well defined through row JMT-2
c
      do 90 jj = 2, jmtm1
c
c  Compute barotropic component of fluxes through walls.
c
         call setvert(jj)
         fxa  = cstr(jj)*dytr(jj)
         fxb  = fxa*cs(jj)
         jjm1 = max((jj-1),1)
c
         do 30 k=1,km
         do 30 i=2,imt
           iu = min(i,imu)
           fuw(i,k)=(ubaro(i-1,jj)*dyu(jj)*dzvqz(i-1,k,0)+
     &               ubaro(i-1,jjm1)*dyu(jjm1)*dzvqz(i-1,k,1))*fxa
           fvn(i,k)=(vbaro(iu,jj)*dxuq(i  ,k)*dzvqz(iu ,k,0)+
     &               vbaro(i-1,jj)*dxuq(i-1,k)*dzvqz(i-1,k,0))*fxb
     &              *dxt4rq(i,k)
  30     continue
c
c  Set W at the bottom.  Kinematic boundary condition
c
         do 40 i=2,imtm1
           w(i,km+1)=c0
  40     continue
c
c  Compute change of W between levels.
c
         do 50 k=1,km
         do 50 i=2,imtm1
           w(i,k)=c2*((fuw(i+1,k)-fuw (i,k))*dxt4rq(i,k)
     &                  + (fvn(i  ,k)-fvst(i,k)) )
  50     continue
c
c  Integrate upwards from the bottom
c
         do 60 k=km,1,-1
         do 60 i=2,imtm1
           w(i,k)=-w(i,k)+w(i,k+1)
  60     continue
c
c  The surface value of W is rescaled to be the initial pressure tendency
c
         do 70 i=2,imtm1
           ptd(i,jj)=gdt*w(i,1)
  70     continue
c
c  Extend to E/W boundaries.
c
#   ifndef cyclic
         ptd(1  ,jj)=ptd(2    ,jj)
         ptd(imt,jj)=ptd(imtm1,jj)
#   else
         ptd(1  ,jj)=ptd(imtm1,jj)
         ptd(imt,jj)=ptd(2    ,jj)
#   endif
c
c  Transfer quantities computed to the north of the present row
c  to be defined to the south in the computation of the next row
c
         fxa=cst(jj)*dyt(jj)*cstr(jj+1)*dytr(jj+1)
         do 80 k=1,km
         do 80 i=1,imt
           fvst(i,k)=fvn(i,k)*fxa
  80     continue
c
  90  continue
c
c  Extend to N/S boundaries.
c
      do 95 i = 1, imt
           ptd(i,1  )=ptd(i,2)
           ptd(i,jmt)=ptd(i,jmtm1)
  95  continue
c
#  endif
# else
c-----------------------------------------------------------------------
c   Initialize pressure tendency by change of tidal elevation.
c-----------------------------------------------------------------------
c
c  Save scaling between surface vertical flux and initial pressure tendency.
c
      gdt = dtsf*grav
c
      do 10 jj = 1, jmt
      do 10 i = 1, imt
           ptd(i,jj)=gdt*dtsrftd(i,jj)
  10  continue
c
# endif
#endif
c-----------------------------------------------------------------------
c  Initialize pressure at time -dt by pressure tendency.
c-----------------------------------------------------------------------
c
      do 100 jj = 1, jmt
      do 100 i = 1, imt
#if !defined nullps & !defined newpsic & !defined psnull
# if surfpress & defined freesurf & (defined rset_dps | defined smallpsdt3 | defined specpsic)
         pb(i,jj) = p(i,jj) - p5*ptd(i,jj)
# else
         pb(i,jj) = p(i,jj) - ptd(i,jj)
# endif
#elif !defined nullps & !defined psnull
         pb(i,jj) = p(i,jj)
#else
# ifdef psnull
         p(i,jj)  = c0
# endif
         ptd(i,jj)= c0
         pb(i,jj) = p(i,jj)
#endif
#if defined tidepredctr
         ptd(i,jj)= c0
#endif
 100  continue
c
#if defined transbc | defined oias
c-----------------------------------------------------------------------
c  Initialize rolling data.
c-----------------------------------------------------------------------
c
c  Compute surface elevation at time -dt
c
      fxa = c1/grav
c
c    On tracer grid.
c
      do 110 jj = 1, jmt
      do 110 i = 1, imt
         etat(i,jj)=fxa*pb(i,jj)
 110  continue
c
c    On velocity grid.
c
      do 130 jj=1,jmtm1
        do 120 i=1,imtm1
          etav(i,jj)=p25*
     &             (etat(i,jj)+etat(i+1,jj)+etat(i,jj+1)+etat(i+1,jj+1))
 120    continue
# ifdef cyclic
        etav(1,jj)=etav(imtm1,jj)
        etav(imt,jj)=etav(2,jj)
# else
        etav(imt,jj)=etav(imtm1,jj)
# endif
 130  continue
      do 140 i=1,imt
        etav(i,jmt)=etav(i,jmtm1)
 140  continue
c
# ifdef transbc
c  Compute water column depth at time -dt, velocity grid.
c
      do 150 jj = 1, jmt
      do 150 i = 1, imt
         hdvcf(i,jj)=hdv(i,jj)+etav(i,jj)
# if defined sprsnrm01 & defined sprsnrm02 & defined sprsnrm05
         hdvcfx(i,jj)=hdvcf(i,jj)*uland(i,jj)
         hdvcfy(i,jj)=hdvcf(i,jj)*vland(i,jj)
#endif
         hdvcf(i,jj)=hdvcf(i,jj)*min(fkmq(i,jj),c1)
 150  continue
c
# endif
#endif
      return
      end
