      subroutine tide_aux
c
c=======================================================================
c                                                                    ===
c  This routine gets the current tidal fields for the specified row. ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  J           Desired row number.                                   ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /FULLWD/                                                          ===
c                                                                    ===
#ifdef resetjulian
c     D0START  Starting time.         (modified Julian day)          ===
#else
c     DSTART   starting time.         (modified Julian day)          ===
#endif
c     TTSEC    current elapsed time.  (s)                            ===
c                                                                    ===
#if defined surfpress & defined freesurf
c  /ONEDIM/                                                          ===
c                                                                    ===
c     CSR     reciprocal cosine metric at V-points.                  ===
c     DXU2R   reciprocal twice zonal spacing, V-boxes.               ===
c     DYU2R   reciprocal twice meridional spacing, V-boxes.          ===
c                                                                    ===
#endif
c  /SCALAR/                                                          ===
c                                                                    ===
c     DTTS     length of timestep on tracer.   (sec)                 ===
#if defined surfpress & defined freesurf
c     GRAV     acceleration due to gravity.    (cm/s^2)              ===
#endif
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /TIDESP/                                                          ===
c                                                                    ===
#if !defined surfpress | !defined freesurf
c     SRFTD    tidal elevation present time step.                    ===
# ifdef tide_zero
c     TUUX     residual tidal stress uux                             ===
c     TVUY     residual tidal stress vuy                             ===
c     TWUZ     residual tidal stress wuz                             ===
c     TUVX     residual tidal stress uvx                             ===
c     TVVY     residual tidal stress vvy                             ===
c     TWVZ     residual tidal stress wvz                             ===
# endif
#else
c     SRFTD    tidal elevation: at T= N-1, N, N+1.                   ===
c     PTDTX    d2PsTide/dtdx: at T= N-1, N, N+1.                     ===
c     PTDTY    d2PsTide/dtdy: at T= N-1, N, N+1.                     ===
c     UBARTD   U barotropic tidal vel.: at T= N-1, N, N+1.           ===
c     VBARTD   V barotropic tidal vel.: at T= N-1, N, N+1.           ===
#endif
c     UTIDE    zonal component of tidal velocity southern row.       ===
c     UTIDEP   zonal component of tidal velocity northern row.       ===
c     VTIDE    meridional component of tidal velocity southern row.  ===
c     VTIDEP   meridional component of tidal velocity northern row.  ===
c                                                                    ===
#if defined surfpress & defined freesurf
c  /BNDATA/                                                          ===
c                                                                    ===
c     PO   surface pressure boundary conditions at TTBDY0.           ===
c                                                                    ===
#endif
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <fullwd.h>
#include <scalar.h>
#include <tidesp.h>
#if defined tide_zero
# include <onedim.h>
#endif
#if defined surfpress & defined freesurf
# include <bndata.h>
# include <onedim.h>
# include <fieldsbar.h>
# include <sinfo.h>
# if defined uvhattran | (defined sprsnrm01 & defined sprsnrm02)
#  include <vertslabs.h>
# endif
# if (defined uvhattran & defined uvhattdsub) | defined tide_pert
#  include <fields.h>
#  include <options.h>
# endif
#endif
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,j,k,n,nocomp,jj
      logical first
      FLOAT
     *      tstart
      FLOAT
     *      t0tid(maxcomp),tptid(maxcomp)
      COMPLEX
     &        cxtime,e_iomt
      COMPLEX
     &        iomega(maxcomp),iphase(maxcomp),sftdcx(imt,jmt,maxcomp),
     &        utidcx(imt,jmt,maxcomp),vtidcx(imt,jmt,maxcomp)
#ifdef tide_pert
     &        ,phsprt(maxcomp)
#endif
c
#if defined tide_zero
      COMPLEX
     &      uval,duyp,duym,duxm,duxp,vval,dvyp,dvym,dvxm,dvxp
c
      integer im1,ip1,jm1,jp1
#endif
#if defined tide_zero | defined tidramp
      FLOAT
     &      ramp
#endif
      FLOAT
     &     ut,vt
#if defined surfpress & defined freesurf
      FLOAT
     &     fxa,fxb
     &     ,fac,fx,fx2
# if defined uvhattdsub
     &     ,fx3,psne,psnw,psse,pssw
# endif
# if !defined fdgtideztd & !defined tidepsic
      FLOAT
     &     dtsrftd(imt,jmt)
# endif
      integer ib,ibb(4)
#endif
#ifdef tide_pert
      FLOAT
     &      ampscal(maxcomp)
#endif
c
      save first,nocomp,tstart,t0tid,tptid,iomega,iphase,sftdcx,utidcx,
     &     vtidcx
#if defined surfpress & defined freesurf
      save ibb
#endif
c
      data first /.true./
#if defined surfpress & defined freesurf
      data ibb/1,1,imt,jmt/
#endif
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c=======================================================================
c  Entry to construct tidal velocities.
c=======================================================================
c
      entry get_tide (j)
c
c-----------------------------------------------------------------------
c  On first call, read tidal data.
c-----------------------------------------------------------------------
c
      if (first) then
#ifndef resetjulian
         tstart=dstart
#else
         tstart=d0start
#endif
         call read_tide (tstart,nocomp,iomega,iphase,t0tid,tptid,sftdcx,
#if !defined tide_pert
     &                   utidcx,vtidcx)
#else
     &                   utidcx,vtidcx,ampscal,phsprt)
#endif
         first = .false.
      end if
c
c-----------------------------------------------------------------------
c  Construct barotropic tidal fields.
c-----------------------------------------------------------------------
c
c  Set time, clear working arrays.
c
      do 10 i=1,imt
         utidep(i,1)=c0
         vtidep(i,1)=c0
         btidep(i  )=c0
  10  continue
c
#if defined tidramp
      ramp = min( max(ttsec,c0)/86400 , c1)      
c
#endif
c  Compute barotropic tidal fields.
c
      do 20 n=1,nocomp
         cxtime = CMPLX ( mod( (t0tid(n)+ttsec), tptid(n)) , c0 )
         e_iomt = exp(iomega(n)*cxtime + iphase(n))
#if defined tidramp
     &            *ramp
#endif
         do 20 i=1,imt
            ut=ReaL(utidcx(i,j+1,n)*e_iomt)
            vt=ReaL(vtidcx(i,j+1,n)*e_iomt)
            utidep(i,1)=utidep(i,1)+ut
            vtidep(i,1)=vtidep(i,1)+vt
            btidep(i  )=btidep(i  )+ut*ut+vt*vt
  20  continue
c
      if (j.eq.1) then
# if defined surfpress & defined freesurf
         itdb = mod(itdb+1,3)
         itd  = mod(itd +1,3)
         itda = mod(itda+1,3)
# endif
         do 30 i=1,imt
            do 25 jj=1,jmt
#if !defined surfpress | !defined freesurf
               srftd(i,jj)=c0
#else
               srftd(i,jj,itda)=c0
               dtsrftd(i,jj)   =c0
               ptdtx(i,jj,itda)=c0
               ptdty(i,jj,itda)=c0
               ubartd(i,jj,itda)=c0
               vbartd(i,jj,itda)=c0
# if defined tidvforc
               dtubartd(i,jj,itda)=c0
               dtvbartd(i,jj,itda)=c0
# endif
#endif
 25         continue
            utide(i,1)=c0
            vtide(i,1)=c0
            btide(i  )=c0
  30     continue
         do 40 n=1,nocomp
            cxtime = CMPLX ( mod( (t0tid(n)+ttsec), tptid(n)), c0 )
            e_iomt  = exp(iomega(n)*cxtime + iphase(n))
#if defined tidramp
     &                *ramp
#endif
            do 40 i=1,imt
               do 35 jj=1,jmt
#if !defined surfpress | !defined freesurf
                  srftd(i,jj)=srftd(i,jj)+ReaL(sftdcx(i,jj,n)*e_iomt)
#else
                  srftd(i,jj,itda) = srftd(i,jj,itda)
     &                               + ReaL(sftdcx(i,jj,n)*e_iomt)
                  dtsrftd(i,jj) = dtsrftd(i,jj)
     &                          + ReaL(iomega(n)*sftdcx(i,jj,n)*e_iomt)
                  ubartd(i,jj,itda) = ubartd(i,jj,itda)
     &                               + ReaL(utidcx(i,jj,n)*e_iomt)
                  vbartd(i,jj,itda) = vbartd(i,jj,itda)
     &                               + ReaL(vtidcx(i,jj,n)*e_iomt)
# if defined tidvforc
                  dtubartd(i,jj,itda) = dtubartd(i,jj,itda)
     &                           + ReaL(iomega(n)*utidcx(i,jj,n)*e_iomt)
                  dtvbartd(i,jj,itda) = dtvbartd(i,jj,itda)
     &                           + ReaL(iomega(n)*vtidcx(i,jj,n)*e_iomt)
# endif
#endif
 35            continue
               ut=ReaL(utidcx(i,j,n)*e_iomt)
               utide(i,1)=utide(i,1)+ut
               vt=ReaL(vtidcx(i,j,n)*e_iomt)
               vtide(i,1)=vtide(i,1)+vt
               btide(i  )=btide(i  )+ut*ut+vt*vt
  40     continue
#if defined surfpress & defined freesurf
         fac = grav*c2dtps
         do 45 jj=1,jmtm1
            fxa = fac*csr(jj)
            fxb = fac*dyu2r(jj)
            do 45 i=1,imtm1
               ptdtx(i,jj,itda) = ((dtsrftd(i+1,jj+1)-dtsrftd(i,jj+1)) +
     &                             (dtsrftd(i+1,jj)-dtsrftd(i,jj)))
     &                                                     *fxa*dxu2r(i)
               ptdty(i,jj,itda) = ((dtsrftd(i+1,jj+1)-dtsrftd(i+1,jj)) +
     &                            (dtsrftd(i,jj+1)-dtsrftd(i,jj))) * fxb
  45     continue
#endif
      end if
#ifndef barotropic
c
c-----------------------------------------------------------------------
c  Spread throughout water column.
c-----------------------------------------------------------------------
c
      do 50 k=2,km
      do 50 i=1,imt
         utidep(i,k)=utidep(i,1)
         vtidep(i,k)=vtidep(i,1)
  50  continue
c
      if (j.eq.1) then
         do 60 k=2,km
            do 60 i=1,imt
               utide(i,k)=utide(i,1)
               vtide(i,k)=vtide(i,1)
 60      continue
      end if
#endif
c
      return
c
#ifdef tide_pert
c=======================================================================
c  Entry to construct tidal perturbation velocities.
c=======================================================================
c
      entry get_perttide
c
c  Update barotropic ICs with tidal perturbations.
c
      do 100 n=1,nocomp
         cxtime = CMPLX ( mod( (t0tid(n)+ttsec), tptid(n)) , c0 )
         e_iomt = ( exp(iomega(n)*cxtime + iphase(n)) -
     &        exp(iomega(n)*cxtime + iphase(n) - phsprt(n))/ampscal(n) )
#if defined tidramp
     &            *ramp
#endif
C         fac = ampscal(n) - c1
         do 100 i=1,imt
         do 100 jj=1,jmt
            p(i,jj) = p(i,jj) + grav*ReaL(sftdcx(i,jj,n)*e_iomt)
            ubaro(i,jj) = ubaro(i,jj) + ReaL(utidcx(i,jj,n)*e_iomt)
            vbaro(i,jj) = vbaro(i,jj) + ReaL(vtidcx(i,jj,n)*e_iomt)
 100  continue
c
#endif
c=======================================================================
c  Entry to construct residual tidal stress.
c=======================================================================
c
      entry tide_stress0 (j)
c
#ifdef tide_zero
      do 200 k=1,km
      do 200 i=1,imt
         tuux(i,k)=c0
         tvuy(i,k)=c0
         twuz(i,k)=c0
         tuvx(i,k)=c0
         tvvy(i,k)=c0
         twvz(i,k)=c0
 200  continue
      jp1=min(j+1,jmt)
      jm1=max(j-1,1)
      ramp=min(ttsec/86400,c1/c2)      
      do 210 n=1,nocomp
         do 210 i=1,imt
            im1=max(i-1,1)
            ip1=min(i+1,imt)
c
            uval=utidcx(i,j,n)
            duyp=conjg( utidcx(i,jp1,n)-uval )*dytr(jp1)
            duym=conjg( uval-utidcx(i,jm1,n) )*dytr(j)
            duxm=conjg( uval-utidcx(im1,j,n) )*dxtr(i)*cstr(j)
            duxp=conjg( utidcx(ip1,j,n)-uval )*dxtr(ip1)*cstr(j)
c
            vval=vtidcx(i,j,n)
            dvyp=conjg( vtidcx(i,jp1,n)-vval )*dytr(jp1)
            dvym=conjg( vval-vtidcx(i,jm1,n) )*dytr(j)
            dvxm=conjg( vval-vtidcx(im1,j,n) )*dxtr(i)*cstr(j)
            dvxp=conjg( vtidcx(ip1,j,n)-vval )*dxtr(ip1)*cstr(j)
c
            tuux(i,1)=tuux(i,1)-p25*ReaL(uval*(duxm+duxp))*ramp
            tvuy(i,1)=tvuy(i,1)-p25*ReaL(vval*(duym+duyp))*ramp
c
            tuvx(i,1)=tuvx(i,1)-p25*ReaL(uval*(dvxm+dvxp))*ramp
            tvvy(i,1)=tvvy(i,1)-p25*ReaL(vval*(dvym+dvyp))*ramp
# if defined show_tide_zero
            if(itt.eq.1) then
               write(99,'(5(1x,e18.7))')
     $              tuux(i,1)/ramp,tvuy(i,1)/ramp,
     $              tuvx(i,1)/ramp,tvvy(i,1)/ramp,
     $              c2*omega*sine(i,j)
            endif
# endif
# if defined tide_zero_clip
            if (abs(tuux(i,1)).gt.c27) then
               tuux(i,1)=c27*tuux(i,1)/abs(tuux(i,1))
               tvuy(i,1)=c27*tvuy(i,1)/abs(tvuy(i,1))
               tuvx(i,1)=c27*tuvx(i,1)/abs(tuvx(i,1))
               tvvy(i,1)=c27*tvvy(i,1)/abs(tvvy(i,1))
            endif
# endif
 210  continue
c
      do 220 k=1,km
         do 220 i=1,imt
            tuux(i,k)=tuux(i,1)
            tvuy(i,k)=tvuy(i,1)
            tuvx(i,k)=tuvx(i,1)
            tvvy(i,k)=tvvy(i,1)
 220  continue
      return
c
#endif
#if defined surfpress & defined freesurf 
c=======================================================================
c  Entry to construct apply tidal elevation as surface pressure BCs.
c=======================================================================
c
      entry bdy_tide_press
c
      do 250 ib=1,4
         if(ib.eq.west.or.ib.eq.east) then
            do 230 i=1,imx(ib)
               po(i,1,ib)=grav*srftd(ibb(ib),i,itda)
 230        continue
         else
            do 240 i=1,imx(ib)
               po(i,1,ib)=grav*srftd(i,ibb(ib),itda)
 240        continue
         endif
 250  continue
      return
c
c=======================================================================
c  Entry to construct apply tidal Uhat BCs.
c=======================================================================
c
      entry bdy_tide_uhat
c
# ifndef try2dtpsic
        if((mix.eq.1).and.eb) then
           fac=c1
        else
           if((mxp.ne.0).or.(mix.ne.0)) then
              fac=p5
           else
              fac=c1
           endif
        endif
        fx2=alphap*fac*c2dtsf
# else
        fx2=alphap*c2dtps
# endif
c
        do 260 jj = 1, jmtm1
# if !defined sprsnrm01 | !defined sprsnrm02
           if (fkmq(1,jj).gt.c0) then
#  ifndef uvhattran
              ubarhat(1,jj) = ubartd(1,jj,itda) + fx2*ptdtx(1,jj,itda)
              vbarhat(1,jj) = vbartd(1,jj,itda) + fx2*ptdty(1,jj,itda)
#  else
#   if !defined uvhattdsub & !defined uvhat2tdsub
              fac           = hdv(1,jj)/hdvcf(1,jj)
              ubarhat(1,jj) = fac*ubartd(1,jj,itda) + fx2*
     &                                                ptdtx(1,jj,itda)
              vbarhat(1,jj) = fac*vbartd(1,jj,itda) + fx2*
     &                                                ptdty(1,jj,itda)
#   elif defined uvhattdsub
              fx3           = c1/hdvcf(1,jj)
              fac           = hdv(1,jj)*fx3
              psnw = (po(jj+1,1,west)-pb(1,jj+1))
              psne = (pbc0(jj+1,2,itbp,west)
     &                       + grav*srftd(2,jj+1,itda)
     &                       + detatby(jj+1,2,itbp,west)*grav
     &                       - pb(2,jj+1))
              pssw = (po(jj,1,west)-pb(1,jj))
              psse = (pbc0(jj,2,itbp,west)
     &                       + grav*srftd(2,jj,itda)
     &                       + detatby(jj,2,itbp,west)*grav
     &                       - pb(2,jj))
              ubarhat(1,jj) = utrano(jj,1,itbp,west)*fx3 +
     &                        fac*ubartd(1,jj,itda) + fx2*
     &                        ((psne-psnw)+(psse-pssw))*csr(jj)*dxu2r(1)
              vbarhat(1,jj) = vtrano(jj,1,itbp,west)*fx3 +
     &                        fac*vbartd(1,jj,itda) + fx2*
     &                        ((psne-psse)+(psnw-pssw))*dyu2r(jj)
              if (iopt(4).eq.7) then
                 ubarhat(1,jj) = ubarhat(1,jj) +dubaro(jj,itbp,west)*fx3
                 vbarhat(1,jj) = vbarhat(1,jj) +dvbaro(jj,itbp,west)*fx3
              end if
#   else
              fx3           = c1/hdvcf(1,jj)
              fac           = hdv(1,jj)*fx3
#   endif
#  endif
           end if
# else
#  if defined  uvhattran & defined uvhattdsub
           if ((uland(1,jj).gt.c0).or.(vland(1,jj).gt.c0)) then
              psnw = (po(jj+1,1,west)-pb(1,jj+1))
              psne = (pbc0(jj+1,2,itbp,west)
     &                       + grav*srftd(2,jj+1,itda)
     &                       + detatby(jj+1,2,itbp,west)*grav
     &                       - pb(2,jj+1))
              pssw = (po(jj,1,west)-pb(1,jj))
              psse = (pbc0(jj,2,itbp,west)
     &                       + grav*srftd(2,jj,itda)
     &                       + detatby(jj,2,itbp,west)*grav
     &                       - pb(2,jj))
           end if
#  endif
           if (uland(1,jj).gt.c0) then
#  ifndef uvhattran
              fac           = hdv(1,jj)/hdvcfx(1,jj)
              ubarhat(1,jj) = ubartd(1,jj,itda) + fx2*ptdtx(1,jj,itda)
#  else
#   ifndef uvhattdsub
              fac           = hdv(1,jj)/hdvcfx(1,jj)
              ubarhat(1,jj) = fac*ubartd(1,jj,itda) + fx2*
     &                                                ptdtx(1,jj,itda)
#   else
              fx3           = c1/hdvcfx(1,jj)
              fac           = hdv(1,jj)*fx3
              ubarhat(1,jj) = utrano(jj,1,itbp,west)*fx3 +
     &                        fac*ubartd(1,jj,itda) + fx2*
     &                        ((psne-psnw)+(psse-pssw))*csr(jj)*dxu2r(1)
              if (iopt(4).eq.7) then
                 ubarhat(1,jj) = ubarhat(1,jj) + dubaro(jj,itbp,west)
              end if
#   endif
#  endif
           end if
           if (vland(1,jj).gt.c0) then
#  ifndef uvhattran
              vbarhat(1,jj) = vbartd(1,jj,itda) + fx2*ptdty(1,jj,itda)
#  else
#   ifndef uvhattdsub
              fac           = hdv(1,jj)/hdvcfy(1,jj)
              vbarhat(1,jj) = fac*vbartd(1,jj,itda) + fx2*
     &                                                ptdty(1,jj,itda)
#   else
              fx3           = c1/hdvcfy(1,jj)
              fac           = hdv(1,jj)*fx3
              vbarhat(1,jj) = vtrano(jj,1,itbp,west)*fx3 +
     &                        fac*vbartd(1,jj,itda) + fx2*
     &                        ((psne-psse)+(psnw-pssw))*dyu2r(jj)
              if (iopt(4).eq.7) then
                 vbarhat(1,jj) = vbarhat(1,jj) + dvbaro(jj,itbp,west)
              end if
#   endif
#  endif
           end if
# endif
c
# if !defined sprsnrm01 | !defined sprsnrm02
           if (fkmq(imu,jj).gt.c0) then
#  ifndef uvhattran
              ubarhat(imu,jj) = ubartd(imu,jj,itda) + fx2*
     &                                                ptdtx(imu,jj,itda)
              vbarhat(imu,jj) = vbartd(imu,jj,itda) + fx2*
     &                                                ptdty(imu,jj,itda)
#  else
#   if !defined uvhattdsub & !defined uvhat2tdsub
              fac             = hdv(imu,jj)/hdvcf(imu,jj)
              ubarhat(imu,jj) = fac*ubartd(imu,jj,itda) + fx2*
     &                                                ptdtx(imu,jj,itda)
              vbarhat(imu,jj) = fac*vbartd(imu,jj,itda) + fx2*
     &                                                ptdty(imu,jj,itda)
#   elif defined uvhattdsub
              fx3           = c1/hdvcf(imu,jj)
              fac             = hdv(imu,jj)*fx3
              psne = (po(jj+1,1,east)-pb(imt,jj+1))
              psnw = (pbc0(jj+1,2,itbp,east)
     &                       + grav*srftd(imtm1,jj+1,itda)
     &                       + detatby(jj+1,2,itbp,east)*grav
     &                       - pb(imtm1,jj+1))
              psse = (po(jj,1,east)-pb(imt,jj))
              pssw = (pbc0(jj,2,itbp,east)
     &                       + grav*srftd(imtm1,jj,itda)
     &                       + detatby(jj,2,itbp,east)*grav
     &                       - pb(imtm1,jj))
              ubarhat(imu,jj) = utrano(jj,1,itbp,east)*fx3 +
     &                          fac*ubartd(imu,jj,itda) + fx2*
     &                      ((psne-psnw)+(psse-pssw))*csr(jj)*dxu2r(imu)
              vbarhat(imu,jj) = vtrano(jj,1,itbp,east)*fx3 +
     &                          fac*vbartd(imu,jj,itda) + fx2*
     &                          ((psne-psse)+(psnw-pssw))*dyu2r(jj)
              if (iopt(4).eq.7) then
                 ubarhat(imu,jj) = ubarhat(imu,jj) +
     &                                          dubaro(jj,itbp,east)*fx3
                 vbarhat(imu,jj) = vbarhat(imu,jj) +
     &                                          dvbaro(jj,itbp,east)*fx3
              end if
#   else
              fx3           = c1/hdvcf(imu,jj)
              fac             = hdv(imu,jj)*fx3
#   endif
#  endif
           end if
# else
#  if defined uvhattran & defined uvhattdsub
           if ((uland(imu,jj).gt.c0).or.(vland(imu,jj).gt.c0)) then
              psne = (po(jj+1,1,east)-pb(imt,jj+1))
              psnw = (pbc0(jj+1,2,itbp,east)
     &                       + grav*srftd(imtm1,jj+1,itda)
     &                       + detatby(jj+1,2,itbp,east)*grav
     &                       - pb(imtm1,jj+1))
              psse = (po(jj,1,east)-pb(imt,jj))
              pssw = (pbc0(jj,2,itbp,east)
     &                       + grav*srftd(imtm1,jj,itda)
     &                       + detatby(jj,2,itbp,east)*grav
     &                       - pb(imtm1,jj))
           end if
#  endif
           if (uland(imu,jj).gt.c0) then
#  ifndef uvhattran
              ubarhat(imu,jj) = ubartd(imu,jj,itda) + fx2*
     &                                                ptdtx(imu,jj,itda)
#  else
#   ifndef uvhattdsub
              fac             = hdv(imu,jj)/hdvcfx(imu,jj)
              ubarhat(imu,jj) = fac*ubartd(imu,jj,itda) + fx2*
     &                                                ptdtx(imu,jj,itda)
#   else
              fx3           = c1/hdvcfx(imu,jj)
              fac             = hdv(imu,jj)*fx3
              ubarhat(imu,jj) = utrano(jj,1,itbp,east)*fx3 +
     &                          fac*ubartd(imu,jj,itda) + fx2*
     &                      ((psne-psnw)+(psse-pssw))*csr(jj)*dxu2r(imu)
              if (iopt(4).eq.7) then
                 ubarhat(imu,jj) = ubarhat(imu,jj) +dubaro(jj,itbp,east)
              end if
#   endif
#  endif
           end if
           if (vland(imu,jj).gt.c0) then
#  ifndef uvhattran
              vbarhat(imu,jj) = vbartd(imu,jj,itda) + fx2*
     &                                                ptdty(imu,jj,itda)
#  else
#   ifndef uvhattdsub
              fac             = hdv(imu,jj)/hdvcfy(imu,jj)
              vbarhat(imu,jj) = fac*vbartd(imu,jj,itda) + fx2*
     &                                                ptdty(imu,jj,itda)
#   else
              fx3           = c1/hdvcfy(imu,jj)
              fac             = hdv(imu,jj)*fx3
              vbarhat(imu,jj) = vtrano(jj,1,itbp,east)*fx3 +
     &                          fac*vbartd(imu,jj,itda) + fx2*
     &                          ((psne-psse)+(psnw-pssw))*dyu2r(jj)
              if (iopt(4).eq.7) then
                 vbarhat(imu,jj) = vbarhat(imu,jj) +dvbaro(jj,itbp,east)
              end if
#   endif
#  endif
           end if
# endif
 260    continue
c
        do 270 i = 1, imu
# if !defined sprsnrm01 | !defined sprsnrm02
           if (fkmq(i,1).gt.c0) then
#  ifndef uvhattran
              ubarhat(i,1) = ubartd(i,1,itda) + fx2*ptdtx(i,1,itda)
              vbarhat(i,1) = vbartd(i,1,itda) + fx2*ptdty(i,1,itda)
#  else
#   if !defined uvhattdsub & !defined uvhat2tdsub
              fac          = hdv(i,1)/hdvcf(i,1)
              ubarhat(i,1) = fac*ubartd(i,1,itda) + fx2*ptdtx(i,1,itda)
              vbarhat(i,1) = fac*vbartd(i,1,itda) + fx2*ptdty(i,1,itda)
#   elif defined uvhattdsub
              fx3           = c1/hdvcf(i,1)
              fac          = hdv(i,1)*fx3
              psnw = (pbc0(i,2,itbp,south)
     &                       + grav*srftd(i,2,itda)
     &                       + detatby(i,2,itbp,south)*grav
     &                       - pb(i,2))
              psne = (pbc0(i+1,2,itbp,south)
     &                       + grav*srftd(i+1,2,itda)
     &                       + detatby(i+1,2,itbp,south)*grav
     &                       - pb(i+1,2))
              pssw = (po(i,1,south) - pb(i,1))
              psse = (po(i+1,1,south) - pb(i+1,1))
              ubarhat(i,1) = utrano(i,1,itbp,south)*fx3 +
     &                       fac*ubartd(i,1,itda) + fx2*
     &                       ((psne-psnw)+(psse-pssw))*csr(1)*dxu2r(i)
              vbarhat(i,1) = vtrano(i,1,itbp,south)*fx3 +
     &                       fac*vbartd(i,1,itda) + fx2*
     &                       ((psne-psse)+(psnw-pssw))*dyu2r(1)
              if (iopt(4).eq.7) then
                 ubarhat(i,1) = ubarhat(i,1) + dubaro(i,itbp,south)*fx3
                 vbarhat(i,1) = vbarhat(i,1) + dvbaro(i,itbp,south)*fx3
              end if
#   else
              fx3           = c1/hdvcf(i,1)
              fac          = hdv(i,1)*fx3
#   endif
#  endif
           end if
# else
#  if defined uvhattran & defined uvhattdsub
           if ((uland(i,1).gt.c0).or.(vland(i,1).gt.c0)) then
              psnw = (pbc0(i,2,itbp,south)
     &                       + grav*srftd(i,2,itda)
     &                       + detatby(i,2,itbp,south)*grav
     &                       - pb(i,2))
              psne = (pbc0(i+1,2,itbp,south)
     &                       + grav*srftd(i+1,2,itda)
     &                       + detatby(i+1,2,itbp,south)*grav
     &                       - pb(i+1,2))
              pssw = (po(i,1,south) - pb(i,1))
              psse = (po(i+1,1,south) - pb(i+1,1))
           end if
#  endif
           if (uland(i,1).gt.c0) then
#  ifndef uvhattran
              ubarhat(i,1) = ubartd(i,1,itda) + fx2*ptdtx(i,1,itda)
#  else
#   ifndef uvhattdsub
              fac          = hdv(i,1)/hdvcfx(i,1)
              ubarhat(i,1) = fac*ubartd(i,1,itda) + fx2*ptdtx(i,1,itda)
#   else
              fx3           = c1/hdvcfx(i,1)
              fac          = hdv(i,1)*fx3
              ubarhat(i,1) = utrano(i,1,itbp,south)*fx3 +
     &                       fac*ubartd(i,1,itda) + fx2*
     &                       ((psne-psnw)+(psse-pssw))*csr(1)*dxu2r(i)
              if (iopt(4).eq.7) then
                 ubarhat(i,1) = ubarhat(i,1) + dubaro(i,itbp,south)
              end if
#   endif
#  endif
           end if
           if (vland(i,1).gt.c0) then
#  ifndef uvhattran
              vbarhat(i,1) = vbartd(i,1,itda) + fx2*ptdty(i,1,itda)
#  else
#   ifndef uvhattdsub
              fac          = hdv(i,1)/hdvcfy(i,1)
              vbarhat(i,1) = fac*vbartd(i,1,itda) + fx2*ptdty(i,1,itda)
#   else
              fx3           = c1/hdvcfy(i,1)
              fac          = hdv(i,1)*fx3
              vbarhat(i,1) = vtrano(i,1,itbp,south)*fx3 +
     &                       fac*vbartd(i,1,itda) + fx2*
     &                       ((psne-psse)+(psnw-pssw))*dyu2r(1)
              if (iopt(4).eq.7) then
                 vbarhat(i,1) = vbarhat(i,1) + dvbaro(i,itbp,south)
              end if
#   endif
#  endif
           end if
# endif
c
# if !defined sprsnrm01 | !defined sprsnrm02
           if (fkmq(i,jmtm1).gt.c0) then
#  ifndef uvhattran
              ubarhat(i,jmtm1) = ubartd(i,jmtm1,itda) + fx2*
     &                                               ptdtx(i,jmtm1,itda)
              vbarhat(i,jmtm1) = vbartd(i,jmtm1,itda) + fx2*
     &                                               ptdty(i,jmtm1,itda)
#  else
#   if !defined uvhattdsub & !defined uvhat2tdsub
              fac              = hdv(i,jmtm1)/hdvcf(i,jmtm1)
              ubarhat(i,jmtm1) = fac*ubartd(i,jmtm1,itda) + fx2*
     &                                               ptdtx(i,jmtm1,itda)
              vbarhat(i,jmtm1) = fac*vbartd(i,jmtm1,itda) + fx2*
     &                                               ptdty(i,jmtm1,itda)
#   elif defined uvhattdsub
              fx3              = c1/hdvcf(i,jmtm1)
              fac              = hdv(i,jmtm1)*fx3
              pssw = (pbc0(i,2,itbp,north)
     &                       + grav*srftd(i,jmtm1,itda)
     &                       + detatby(i,2,itbp,north)*grav
     &                       - pb(i,jmtm1))
              psse = (pbc0(i+1,2,itbp,north)
     &                       + grav*srftd(i+1,jmtm1,itda)
     &                       + detatby(i+1,2,itbp,north)*grav
     &                       - pb(i+1,jmtm1))
              psnw = (po(i,1,north) - pb(i,jmt))
              psne = (po(i+1,1,north) - pb(i+1,jmt))
              ubarhat(i,jmtm1) = utrano(i,1,itbp,north)*fx3 +
     &                           fac*ubartd(i,jmtm1,itda) + fx2*
     &                     ((psne-psnw)+(psse-pssw))*csr(jmtm1)*dxu2r(i)
              vbarhat(i,jmtm1) = vtrano(i,1,itbp,north)*fx3 +
     &                           fac*vbartd(i,jmtm1,itda) + fx2*
     &                            ((psne-psse)+(psnw-pssw))*dyu2r(jmtm1)
              if (iopt(4).eq.7) then
                 ubarhat(i,jmtm1) = ubarhat(i,jmtm1) +
     &                                          dubaro(i,itbp,north)*fx3
                 vbarhat(i,jmtm1) = vbarhat(i,jmtm1) +
     &                                          dvbaro(i,itbp,north)*fx3
              end if
#   else
              fx3              = c1/hdvcf(i,jmtm1)
              fac              = hdv(i,jmtm1)*fx3
#   endif
#  endif
           end if
# else
#  if defined uvhattran & defined uvhattdsub
           if ((uland(i,jmtm1).gt.c0).or.(vland(i,jmtm1).gt.c0)) then
              pssw = (pbc0(i,2,itbp,north)
     &                       + grav*srftd(i,jmtm1,itda)
     &                       + detatby(i,2,itbp,north)*grav
     &                       - pb(i,jmtm1))
              psse = (pbc0(i+1,2,itbp,north)
     &                       + grav*srftd(i+1,jmtm1,itda)
     &                       + detatby(i+1,2,itbp,north)*grav
     &                       - pb(i+1,jmtm1))
              psnw = (po(i,1,north) - pb(i,jmt))
              psne = (po(i+1,1,north) - pb(i+1,jmt))
           end if
#  endif
           if (uland(i,jmtm1).gt.c0) then
#  ifndef uvhattran
              ubarhat(i,jmtm1) = ubartd(i,jmtm1,itda) + fx2*
     &                                               ptdtx(i,jmtm1,itda)
#  else
#   ifndef uvhattdsub
              fac              = hdv(i,jmtm1)/hdvcfx(i,jmtm1)
              ubarhat(i,jmtm1) = fac*ubartd(i,jmtm1,itda) + fx2*
     &                                               ptdtx(i,jmtm1,itda)
#   else
              fx3              = c1/hdvcfx(i,jmtm1)
              fac              = hdv(i,jmtm1)*fx3
              ubarhat(i,jmtm1) = utrano(i,1,itbp,north)*fx3 +
     &                           fac*ubartd(i,jmtm1,itda) + fx2*
     &                     ((psne-psnw)+(psse-pssw))*csr(jmtm1)*dxu2r(i)
              if (iopt(4).eq.7) then
                 ubarhat(i,jmtm1)=ubarhat(i,jmtm1) +dubaro(i,itbp,north)
              end if
#   endif
#  endif
           end if
           if (vland(i,jmtm1).gt.c0) then
#  ifndef uvhattran
              vbarhat(i,jmtm1) = vbartd(i,jmtm1,itda) + fx2*
     &                                               ptdty(i,jmtm1,itda)
#  else
#   ifndef uvhattdsub
              fac              = hdv(i,jmtm1)/hdvcfy(i,jmtm1)
              vbarhat(i,jmtm1) = fac*vbartd(i,jmtm1,itda) + fx2*
     &                                               ptdty(i,jmtm1,itda)
#   else
              fx3              = c1/hdvcfy(i,jmtm1)
              fac              = hdv(i,jmtm1)*fx3
              vbarhat(i,jmtm1) = vtrano(i,1,itbp,north)*fx3 +
     &                           fac*vbartd(i,jmtm1,itda) + fx2*
     &                            ((psne-psse)+(psnw-pssw))*dyu2r(jmtm1)
              if (iopt(4).eq.7) then
                 vbarhat(i,jmtm1)=vbarhat(i,jmtm1) +dvbaro(i,itbp,north)
              end if
#   endif
#  endif
           end if
# endif
 270    continue
c
      return
#endif
c
c=======================================================================
c  Entry to reset barotropic tidal fields.
c=======================================================================
c
      entry tid_bar_rset
c
c      write (6,*) 'TID_BAR_RSET: setting tides for ttsec=',ttsec
c      call flush (6)
# if defined surfpress & defined freesurf
      itdb = mod(itdb+1,3)
      itd  = mod(itd +1,3)
      itda = mod(itda+1,3)
# endif
      do 300, jj=1,jmt
      do 300, i=1,imt
#if !defined surfpress | !defined freesurf
            srftd(i,jj)=c0
#else
            srftd(i,jj,itda)=c0
            dtsrftd(i,jj)   =c0
            ptdtx(i,jj,itda)=c0
            ptdty(i,jj,itda)=c0
            ubartd(i,jj,itda)=c0
            vbartd(i,jj,itda)=c0
# if defined tidvforc
            dtubartd(i,jj,itda)=c0
            dtvbartd(i,jj,itda)=c0
# endif
#endif
 300  continue
      do 310, n=1,nocomp
         cxtime = CMPLX ( mod( (t0tid(n)+ttsec), tptid(n)), c0 )
         e_iomt  = exp(iomega(n)*cxtime + iphase(n))
#if defined tidramp
     &                *ramp
#endif
         do 310, jj=1,jmt
         do 310, i=1,imt
#if !defined surfpress | !defined freesurf
               srftd(i,jj)=srftd(i,jj)+ReaL(sftdcx(i,jj,n)*e_iomt)
#else
               srftd(i,jj,itda) = srftd(i,jj,itda)
     &                               + ReaL(sftdcx(i,jj,n)*e_iomt)
               dtsrftd(i,jj) = dtsrftd(i,jj)
     &                          + ReaL(iomega(n)*sftdcx(i,jj,n)*e_iomt)
               ubartd(i,jj,itda) = ubartd(i,jj,itda)
     &                            + ReaL(utidcx(i,jj,n)*e_iomt)
               vbartd(i,jj,itda) = vbartd(i,jj,itda)
     &                            + ReaL(vtidcx(i,jj,n)*e_iomt)
# if defined tidvforc
               dtubartd(i,jj,itda) = dtubartd(i,jj,itda)
     &                           + ReaL(iomega(n)*utidcx(i,jj,n)*e_iomt)
               dtvbartd(i,jj,itda) = dtvbartd(i,jj,itda)
     &                           + ReaL(iomega(n)*vtidcx(i,jj,n)*e_iomt)
# endif
#endif
 310  continue
#if defined surfpress & defined freesurf
      do 320, jj=1,jmtm1
         fxa = grav*csr(jj)
         fxb = grav*dyu2r(jj)
         do 320, i=1,imtm1
            ptdtx(i,jj,itda) = ((dtsrftd(i+1,jj+1)-dtsrftd(i,jj+1)) +
     &                          (dtsrftd(i+1,jj)-dtsrftd(i,jj)))
     &                                                  *fxa*dxu2r(i)
            ptdty(i,jj,itda) = ((dtsrftd(i+1,jj+1)-dtsrftd(i+1,jj)) +
     &                         (dtsrftd(i,jj+1)-dtsrftd(i,jj))) * fxb
 320  continue
#endif
c
      return
c
      end
